@@ -243,6 +243,7 @@ def list_queue(speaker, action, args, soco_function, use_local_speaker_list):
                 error_and_exit(
                     "Track number {} is out of queue range".format(track_number)
                 )
+                return False
             queue = [queue[track_number - 1]]
         except ValueError:
             parameter_type_error(action, "integer")
@@ -1334,6 +1335,7 @@ def wait_stop_core(speaker, not_paused=False):
         sub = speaker.avTransport.subscribe(auto_renew=True)
     except Exception as e:
         error_and_exit("Exception {}".format(e))
+        return False
 
     while True:
         try:
@@ -1372,6 +1374,7 @@ def wait_stopped_for_core(speaker, action, duration_arg, not_paused=False):
         sub = speaker.avTransport.subscribe(auto_renew=True)
     except Exception as e:
         error_and_exit("Exception {}".format(e))
+        return False
 
     playing_states = ["PLAYING", "TRANSITIONING"]
     if not_paused:
@@ -1454,6 +1457,7 @@ def wait_start(speaker, action, args, soco_function, use_local_speaker_list):
         sub = speaker.avTransport.subscribe(auto_renew=True)
     except Exception as e:
         error_and_exit("Exception {}".format(e))
+        return False
     while True:
         try:
             event = sub.events.get(timeout=1.0)
@@ -2056,6 +2060,66 @@ def pauseplay(speaker, action, args, soco_function, use_local_speaker_list):
     return True
 
 
+@zero_parameters
+def available_actions(speaker, action, args, soco_function, use_local_speaker_list):
+    """Determine the currently available playback control options."""
+    actions = speaker.avTransport.GetCurrentTransportActions([("InstanceID", 0)])[
+        "Actions"
+    ]
+    actions = actions.replace("X_DLNA_", "")
+    print("Currently available playback actions: {}".format(actions))
+    return True
+
+
+@zero_parameters
+def wait_end_track(speaker, action, args, soco_function, use_local_speaker_list):
+    """Wait for the end of the current track, or until playback stops/pauses."""
+
+    try:
+        sub = speaker.avTransport.subscribe(auto_renew=True)
+        logging.info(
+            "Subscribing to transport events from {}".format(speaker.player_name)
+        )
+    except Exception as e:
+        error_and_exit("Exception {}".format(e))
+        return False
+
+    initial_track = None
+
+    while True:
+        try:
+            event = sub.events.get(timeout=1.0)
+            logging.info("Transport event received")
+
+            # The code below never works; retain for possible future use
+            #
+            # info = speaker.get_current_track_info()
+            # position = convert_to_seconds(info["position"])
+            # duration = convert_to_seconds(info["duration"])
+            # logging.info("Position = {}, duration = {}".format(position, duration))
+            #
+            # if duration - position == 0:
+            #     logging.info("Track duration expired")
+            #     event_unsubscribe(sub)
+            #     return True
+
+            if event.variables["transport_state"] not in ["PLAYING", "TRANSITIONING"]:
+                logging.info("Speaker is not playing")
+                event_unsubscribe(sub)
+                return True
+
+            if not initial_track:
+                initial_track = event.variables["current_track"]
+
+            elif event.variables["current_track"] != initial_track:
+                logging.info("Track number has changed")
+                event_unsubscribe(sub)
+                return True
+
+        except Empty:
+            pass
+
+
 def process_action(speaker, action, args, use_local_speaker_list=False):
     sonos_function = actions.get(action, None)
     if sonos_function:
@@ -2386,4 +2450,6 @@ actions = {
     "groupstatus": SonosFunction(groupstatus),
     "pauseplay": SonosFunction(pauseplay, "", True),
     "playpause": SonosFunction(pauseplay, "", True),
+    "available_actions": SonosFunction(available_actions, "", True),
+    "wait_end_track": SonosFunction(wait_end_track, "", True),
 }
